#include <xen/config.h>
#include <xen/init.h>
#include <xen/lib.h>
#include <xen/string.h>
#include <xen/sched.h>
#include <xen/smp.h>

#include <asm/cpu.h>
#include <asm/cpu-info.h>
#include <asm/mipsregs.h>
#include <asm/netl/platform.h>

#define PROCESSOR_ID_MAX_LEN 6

char cpu_model_info[100]={'R','M','I'};

void (*cpu_wait)(void);

#define is_xls() (0)

static int get_xls_proc_name(void)
{
        int processor_id = ((read_c0_prid() & 0xff00) >> 8);
        switch ( processor_id ) {
        case CHIP_PROCESSOR_ID_XLS_608:
                safe_strcpy(cpu_model_info,"XLS608");
                break;
        case CHIP_PROCESSOR_ID_XLS_408:
                safe_strcpy(cpu_model_info,"XLS408");
                break;
        case CHIP_PROCESSOR_ID_XLS_404:
                safe_strcpy(cpu_model_info,"XLS404");
                break;
        case CHIP_PROCESSOR_ID_XLS_208:
                safe_strcpy(cpu_model_info,"XLS208");
                break;
        case CHIP_PROCESSOR_ID_XLS_204:
                safe_strcpy(cpu_model_info,"XLS204");
                break;
        default:
                safe_strcpy(cpu_model_info,"XLS???");
                return -1;
        }
        return 0;
}

static int get_xlr_proc_name(void )
{
	int processor_id = ((read_c0_prid() & 0xff00) >> 8);
	if ( xlr_revision_c()) {
		switch ( processor_id ) {
		case CHIP_PROCESSOR_ID_XLR_C_308:
			safe_strcpy(cpu_model_info,"XLR308");
			break;
		case CHIP_PROCESSOR_ID_XLR_C_508:
			safe_strcpy(cpu_model_info,"XLR508");
			break;
		case CHIP_PROCESSOR_ID_XLR_C_516:
			safe_strcpy(cpu_model_info,"XLR516");
			break;
		case CHIP_PROCESSOR_ID_XLR_C_532:
			safe_strcpy(cpu_model_info,"XLR532");
			break;
		case CHIP_PROCESSOR_ID_XLR_C_716:
			safe_strcpy(cpu_model_info,"XLR716");
			break;
		case CHIP_PROCESSOR_ID_XLR_C_732:
			safe_strcpy(cpu_model_info,"XLR732");	
			break;
		default:
			safe_strcpy(cpu_model_info,"XLR???");
                	return -1;
		}
	} else {
		switch ( processor_id ) {
		case CHIP_PROCESSOR_ID_XLR_B_308:
			safe_strcpy(cpu_model_info,"XLR308");
			break;
		case CHIP_PROCESSOR_ID_XLR_B_508:
			safe_strcpy(cpu_model_info,"XLR508");
			break;
		case CHIP_PROCESSOR_ID_XLR_B_516:
			safe_strcpy(cpu_model_info,"XLR516");
			break;
		case CHIP_PROCESSOR_ID_XLR_B_532:
			safe_strcpy(cpu_model_info,"XLR532");
			break;
		case CHIP_PROCESSOR_ID_XLR_B_716:
			safe_strcpy(cpu_model_info,"XLR716");
			break;
		case CHIP_PROCESSOR_ID_XLR_B_732:
			safe_strcpy(cpu_model_info,"XLR732");	
			break;
		default:
			safe_strcpy(cpu_model_info,"XLR???");
			return -1;
		}
	}
        return 0;
}

static int get_xlr_revision(void )
{
        int revision =  xlr_revision();
        switch( revision ) {
        case XLR_REVISION_A0:
                strcpy(cpu_model_info+PROCESSOR_ID_MAX_LEN," Rev A0");
                break;
        case XLR_REVISION_A1:
                strcpy(cpu_model_info+PROCESSOR_ID_MAX_LEN," Rev A1");
                break;
        case XLR_REVISION_B0:
                strcpy(cpu_model_info+PROCESSOR_ID_MAX_LEN," Rev B0");
                break;
        case XLR_REVISION_B1:
                strcpy(cpu_model_info+ PROCESSOR_ID_MAX_LEN," Rev B1");
                break;
        case XLR_REVISION_B2:
                strcpy(cpu_model_info+ PROCESSOR_ID_MAX_LEN," Rev B2");
                break;
        case XLR_REVISION_C0:
                strcpy(cpu_model_info + PROCESSOR_ID_MAX_LEN," Rev C0");
                break;
        case XLR_REVISION_C1:
                strcpy(cpu_model_info+PROCESSOR_ID_MAX_LEN," Rev C1");
                break;
        case XLR_REVISION_C2:
                strcpy(cpu_model_info+PROCESSOR_ID_MAX_LEN," Rev C2");
                break;
        case XLR_REVISION_C3:
                strcpy(cpu_model_info+PROCESSOR_ID_MAX_LEN," Rev C3");
                break;
        case XLR_REVISION_C4:
                strcpy(cpu_model_info+PROCESSOR_ID_MAX_LEN," Rev C4");
                break;
        default:
                strcpy(cpu_model_info+PROCESSOR_ID_MAX_LEN,"???");
                return -1;
        }
        return 0;
}

static int get_xlp_revision(void )
{
        int revision =  xlr_revision();
        switch( revision ) {
        case XLP_REVISION_A0:
                strcpy(cpu_model_info+PROCESSOR_ID_MAX_LEN," Rev A0");
                break;
	default:
                strcpy(cpu_model_info+PROCESSOR_ID_MAX_LEN,"???");
                return -1;
	}
	return 0;
}

static int get_xlp_proc_name(void )
{
        strcpy(cpu_model_info,"XLP832");	
        return 0;
}

static char unknown_isa[] __initdata = KERN_ERR \
	"Unsupported ISA type, c0.config0: %d.";

static inline unsigned int decode_config0(struct cpuinfo_mips *c)
{
	unsigned int config0;
	int isa;

	config0 = read_c0_config();

	if (((config0 & MIPS_CONF_MT) >> 7) == 1)
		c->options |= MIPS_CPU_TLB;
	isa = (config0 & MIPS_CONF_AT) >> 13;
	switch (isa) {
	case 0:
		switch ((config0 & MIPS_CONF_AR) >> 10) {
		case 0:
			c->isa_level = MIPS_CPU_ISA_M32R1;
			break;
		case 1:
			c->isa_level = MIPS_CPU_ISA_M32R2;
			break;
		default:
			goto unknown;
		}
		break;
	case 2:
		switch ((config0 & MIPS_CONF_AR) >> 10) {
		case 0:
			c->isa_level = MIPS_CPU_ISA_M64R1;
			break;
		case 1:
			c->isa_level = MIPS_CPU_ISA_M64R2;
			break;
		default:
			goto unknown;
		}
		break;
	default:
		goto unknown;
	}

	return config0 & MIPS_CONF_M;

unknown:
	panic(unknown_isa, config0);
	while (1)
		;
}

static inline unsigned int decode_config1(struct cpuinfo_mips *c)
{
	unsigned int config1;

	config1 = read_c0_config1();

	if (config1 & MIPS_CONF1_MD)
		c->ases |= MIPS_ASE_MDMX;
	if (config1 & MIPS_CONF1_WR)
		c->options |= MIPS_CPU_WATCH;
	if (config1 & MIPS_CONF1_CA)
		c->ases |= MIPS_ASE_MIPS16;
	if (config1 & MIPS_CONF1_EP)
		c->options |= MIPS_CPU_EJTAG;
	if (config1 & MIPS_CONF1_FP) {
		c->options |= MIPS_CPU_FPU;
		c->options |= MIPS_CPU_32FPR;
	}

	return config1 & MIPS_CONF_M;
}

static inline unsigned int decode_config2(struct cpuinfo_mips *c)
{
	unsigned int config2;

	config2 = read_c0_config2();

	if (config2 & MIPS_CONF2_SL)
		c->scache.flags &= ~MIPS_CACHE_NOT_PRESENT;

	return config2 & MIPS_CONF_M;
}

static inline unsigned int decode_config3(struct cpuinfo_mips *c)
{
	unsigned int config3;

	config3 = read_c0_config3();

	if (config3 & MIPS_CONF3_SM)
		c->ases |= MIPS_ASE_SMARTMIPS;
	if (config3 & MIPS_CONF3_DSP)
		c->ases |= MIPS_ASE_DSP;
	if (config3 & MIPS_CONF3_VINT)
		c->options |= MIPS_CPU_VINT;
	if (config3 & MIPS_CONF3_VEIC)
		c->options |= MIPS_CPU_VEIC;
	if (config3 & MIPS_CONF3_MT)
	        c->ases |= MIPS_ASE_MIPSMT;
	if (config3 & MIPS_CONF3_ULRI)
		c->options |= MIPS_CPU_ULRI;

	return config3 & MIPS_CONF_M;
}

static inline void decode_config6(struct cpuinfo_mips *c)
{
	unsigned int config6;

	config6 = read_c0_config6();

	if (cpu_has_tlb) {
		c->tlbsize = ((config6 & MIPS_CONF6_TLBS) >> 16) + 1;
	}

}

static void __init decode_configs(struct cpuinfo_mips *c)
{
	/* MIPS32 or MIPS64 compliant CPU.  */
	c->options = MIPS_CPU_4KEX | MIPS_CPU_4K_CACHE | MIPS_CPU_COUNTER |
	             MIPS_CPU_DIVEC | MIPS_CPU_LLSC | MIPS_CPU_MCHECK;

	c->scache.flags = MIPS_CACHE_NOT_PRESENT;

	/* Read Config registers.  */
	if (!decode_config0(c))
		return;			/* actually worth a panic() */
	if (!decode_config1(c))
		return;
	if (!decode_config2(c))
		return;
	decode_config3(c);
	decode_config6(c);
}

static __inline__ void cpu_probe_netl(struct cpuinfo_mips *c)
{
	decode_configs(c);

	c->options = (MIPS_CPU_TLB     |
		      MIPS_CPU_4KEX    |
		      MIPS_CPU_COUNTER |
		      MIPS_CPU_DIVEC   |
		      MIPS_CPU_WATCH   |
		      MIPS_CPU_EJTAG   |
		      MIPS_CPU_PHOENIX_CACHE |
		      MIPS_CPU_LLSC);

	switch (c->processor_id & 0xff00) {

	case CHIP_PROCESSOR_ID_XLP_A_832:
	{
		/* XLP */

		c->cputype = CPU_XLP;

		get_xlp_proc_name();

		get_xlp_revision();

		c->isa_level = MIPS_CPU_ISA_M64R2;
		c->options |= (MIPS_CPU_FPU | MIPS_CPU_ULRI);

		// printk("[%s@%d] Enabling XLP CPU\n", __FUNCTION__, __LINE__);
	}
	break;
	default:
	{
		/* XLR/XLS */

		c->cputype = CPU_XLR;

		if (is_xls())
			get_xls_proc_name();
		else
			get_xlr_proc_name();

		get_xlr_revision();

		c->isa_level = MIPS_CPU_ISA_M64R1;

		printk("[%s@%d] Enabling XLR/XLS CPU\n", __FUNCTION__, __LINE__);
	}
	break;

	}
}

static void xlp_wait(void)
{
	__asm__(
		".set push\n\t"
		".set\tmips3\n\t"
		".set noreorder\n\t"		
		"c0 0x60\n\t"
		".set pop\n\t"
		);
}

void cpu_probe(void)
{
	struct cpuinfo_mips *c = &current_cpu_data;

	c->processor_id	= PRID_IMP_UNKNOWN;
	c->fpu_id	= FPIR_IMP_NONE;
	c->cputype	= CPU_UNKNOWN;

	c->processor_id = read_c0_prid();
	switch (c->processor_id & 0xff0000) {
	case PRID_COMP_NETL:
		cpu_wait = xlp_wait;
		cpu_probe_netl(c);
		break;
	default:
		c->cputype = CPU_UNKNOWN;
	}
#if 0
	/*
	 * FIXME
	 */
	if (c->options & MIPS_CPU_FPU) {
		c->fpu_id = cpu_get_fpu_id();

		if (c->isa_level == MIPS_CPU_ISA_M32R1 ||
		    c->isa_level == MIPS_CPU_ISA_M32R2 ||
		    c->isa_level == MIPS_CPU_ISA_M64R1 ||
		    c->isa_level == MIPS_CPU_ISA_M64R2) {
			if (c->fpu_id & MIPS_FPIR_3D)
				c->ases |= MIPS_ASE_MIPS3D;
		}
	}
#endif
}

