#include <xen/config.h>
#include <xen/init.h>
#include <xen/console.h>
#include <asm/netl/asmmacro.h>
#include <asm/netl/pic.h>

__init int pic_init(void)
{
	register_t *mmio = mmio_address(PHOENIX_IO_PIC_OFFSET);
	int i = 0;
	int level;
	uint32_t thread_mask = (1 << processor_id());

	for (i = 0; i < PIC_NUM_IRTS; i++) {
		level = PIC_IRQ_IS_EDGE_TRIGGERED(i);

		/* Bind all PIC irqs to boot cpu */
#if defined(XLP_SIM)
		mmio = (register_t *) XLP_IO_PIC_OFFSET;
		/* Use local scheduling and high polarity for all IRTs
		 * Invalidate all IRTs, by default
		 */
		pic_write_irt(i, 0, 0, 1, irt_to_irq(i), 1, 0, thread_mask);

		/* Use PIC Timer 6 as a free running counter */
		write_register_dw(mmio, PIC_TIMER_6_MAXVAL, 0xffffffffffffffffUL);
		
		/* enable the timer */
		write_register_dw(mmio, PIC_CTRL, ((1 << (10 + 6)) | read_register_dw(mmio, PIC_CTRL)));
#else
		phoenix_write_reg(mmio, PIC_IRT_0_BASE + i, thread_mask);

		/* Use local scheduling and high polarity for all IRTs
		 * Invalidate all IRTs, by default
		 */
		phoenix_write_reg(mmio, PIC_IRT_1_BASE + i, (level << 30) | (1 << 6) | (PIC_IRQ_BASE + i));
#endif
	}

	return 0;
}
