/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1998, 2002 Ralf Baechle
 * Copyright (C) 1999 Silicon Graphics, Inc.
 */
#ifndef _MIPS_CURRENT_H
#define _MIPS_CURRENT_H

#include <xen/config.h>
#include <xen/percpu.h>
#include <public/xen.h>
#include <asm/page.h>
#include <asm/processor.h>
#include <asm/thread_info.h>

struct vcpu;

static inline struct vcpu *get_current(void)
{
	return current_thread_info()->task;
}
#define set_current(vcpu) \
do { \
	current_thread_info()->task = vcpu; \
} while (0)

static inline struct cpu_user_regs *guest_cpu_user_regs(void)
{
	return (struct cpu_user_regs *)((char *)current_thread_info() + THREAD_SIZE - sizeof(struct cpu_user_regs));
}

#define current get_current()

/*
 * Which VCPU's state is currently running on each CPU?
 * This is not necesasrily the same as 'current' as a CPU may be
 * executing a lazy state switch.
 */
DECLARE_PER_CPU(struct vcpu *, curr_vcpu);

#define task_thread_info(task) (task)->arch.thread_info
#define task_stack_page(task) ((void*)((task)->arch.thread_info))

#endif /* _ASM_CURRENT_H */
