/************************************************************************

  Copyright 2003-2006 Raza Microelectronics, Inc.(RMI).

  This is a derived work from software originally provided by the external
  entity identified below. The licensing terms and warranties specified in
  the header of the original work apply to this derived work.

  Contribution by RMI:

 *****************************#RMI_1#************************************/

/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1994 - 1999, 2000, 03 Ralf Baechle
 * Copyright (C) 1999, 2000 Silicon Graphics, Inc.
 */
#ifndef _ASM_PAGE_H
#define _ASM_PAGE_H

#include <xen/config.h>

#ifndef __ASSEMBLY__

#include <xen/lib.h>
#include <asm/types.h>
#include <spaces.h>
#include <asm/mipsregs.h>

#endif

/*
 * PAGE_SHIFT determines the page size
 */
#ifdef CONFIG_XEN_PAGE_SIZE_4KB
#define PAGE_SHIFT	12
#endif
#ifdef CONFIG_XEN_PAGE_SIZE_8KB
#define PAGE_SHIFT	13
#endif
#ifdef CONFIG_XEN_PAGE_SIZE_16KB
#define PAGE_SHIFT	14
#endif
#ifdef CONFIG_XEN_PAGE_SIZE_64KB
#define PAGE_SHIFT	16
#endif

#define DOM_PAGE_SHIFT 16

#if DOM_PAGE_SHIFT == 22
#define PM_DOM_MASK    PM_4M
#elif DOM_PAGE_SHIFT == 16
#define PM_DOM_MASK    PM_64K
#elif DOM_PAGE_SHIFT == 12
#define PM_DOM_MASK    PM_4K
#else
#error 'Unsupported domain page size'
#endif

#ifndef __ASSEMBLY__
#define PAGE_SIZE	(1UL << PAGE_SHIFT)
#define DOM_PAGE_SIZE	(1UL << DOM_PAGE_SHIFT)
#else
#define PAGE_SIZE	(1 << PAGE_SHIFT)
#define DOM_PAGE_SIZE	(1 << DOM_PAGE_SHIFT)
#endif

#define PAGE_MASK       (~((1 << PAGE_SHIFT) - 1))
#define DOM_PAGE_MASK       (~((1 << DOM_PAGE_SHIFT) - 1))

#ifndef __ASSEMBLY__

/*
 * This gives the physical RAM offset.
 */
#ifndef PHYS_OFFSET
#define PHYS_OFFSET		0UL
#endif

/*
 * It's normally defined only for FLATMEM config but it's
 * used in our early mem init code for all memory models.
 * So always define it.
 */
#define ARCH_PFN_OFFSET		PFN_UP(PHYS_OFFSET)

#include <asm/pfn.h>
#include <asm/io.h>

extern void clear_page(void * page);
extern void copy_page(void * to, void * from);

extern unsigned long shm_align_mask;

static inline unsigned long pages_do_alias(unsigned long addr1,
	unsigned long addr2)
{
	return (addr1 ^ addr2) & shm_align_mask;
}

struct page;

static inline void clear_user_page(void *addr, unsigned long vaddr,
	struct page *page)
{
#ifdef CONFIG_RMI_PHOENIX
	extern void phoenix_flush_dcache_page(struct page *page);
#else
	extern void (*flush_data_cache_page)(unsigned long addr);
#endif

	clear_page(addr);
#ifdef CONFIG_RMI_PHOENIX
	phoenix_flush_dcache_page(page);
#else
	if (pages_do_alias((unsigned long) addr, vaddr & PAGE_MASK))
		flush_data_cache_page((unsigned long)addr);
#endif
}

#ifdef CONFIG_RMI_PHOENIX
static inline void copy_user_page(void *vto, void *vfrom, unsigned long vaddr,
		    struct page *to)
{
	extern void phoenix_flush_dcache_page(struct page *page);

	copy_page(vto, vfrom);
	phoenix_flush_dcache_page(to);
}
#else
extern void copy_user_page(void *vto, void *vfrom, unsigned long vaddr,
	struct page *to);
#endif
struct vm_area_struct;
extern void copy_user_highpage(struct page *to, struct page *from,
	unsigned long vaddr, struct vm_area_struct *vma);

#define __HAVE_ARCH_COPY_USER_HIGHPAGE

/*
 * These are used to make use of C type-checking..
 */
#ifdef CONFIG_XEN_64BIT_PHYS_ADDR
#ifdef CONFIG_CPU_MIPS32
typedef struct { unsigned long pte_low, pte_high; } pte_t;
#define pte_val(x)    ((x).pte_low | ((unsigned long long)(x).pte_high << 32))
#define __pte(x)      ({ pte_t __pte = {(x), ((unsigned long long)(x)) >> 32}; __pte; })
#else
typedef struct { unsigned long long pte; } pte_t;
#define pte_val(x)	((x).pte)
#define __pte(x)	((pte_t) { (x) } )
#endif
#else
typedef struct { unsigned long pte; } pte_t;
#define pte_val(x)	((x).pte)
#define __pte(x)	((pte_t) { (x) } )
#endif

/*
 * For 3-level pagetables we defines these ourselves, for 2-level the
 * definitions are supplied by <asm-generic/pgtable-nopmd.h>.
 */
#ifndef CONFIG_XEN_2LEVEL_PGTABLE

typedef struct { unsigned long pmd; } pmd_t;
#define pmd_val(x)     ((x).pmd)
#define __pmd(x)       ((pmd_t) { (x) } )

#endif


/*
 * Right now we don't support 4-level pagetables, so all pud-related
 * definitions come from <asm-generic/pgtable-nopud.h>.
 */

/*
 * Finall the top of the hierarchy, the pgd
 */
typedef struct { unsigned long pgd; } pgd_t;
#define pgd_val(x)	((x).pgd)
#define __pgd(x)	((pgd_t) { (x) } )

/*
 * Manipulate page protection bits
 */
typedef struct { unsigned long pgprot; } pgprot_t;
#define pgprot_val(x)	((x).pgprot)
#define __pgprot(x)	((pgprot_t) { (x) } )

/*
 * On R4000-style MMUs where a TLB entry is mapping a adjacent even / odd
 * pair of pages we only have a single global bit per pair of pages.  When
 * writing to the TLB make sure we always have the bit set for both pages
 * or none.  This macro is used to access the `buddy' of the pte we're just
 * working on.
 */
#define ptep_buddy(x)	((pte_t *)((unsigned long)(x) ^ sizeof(pte_t)))

#endif /* !__ASSEMBLY__ */

/* to align the pointer to the (next) page boundary */
#define PAGE_ALIGN(addr)	(((addr) + PAGE_SIZE - 1) & PAGE_MASK)

/*
 * __pa()/__va() should be used only during mem init.
 */
#if defined(CONFIG_XEN_64BIT) && !defined(CONFIG_BUILD_ELF64)
#define __pa_page_offset(x)	((unsigned long)(x) < CKSEG0 ? PAGE_OFFSET : CKSEG0)
#else
#define __pa_page_offset(x)	PAGE_OFFSET
#endif

#define __pa(x)		((unsigned long)(x) - __pa_page_offset(x) + PHYS_OFFSET)
#define __va(x)		((void *)((unsigned long)(x) + PAGE_OFFSET - PHYS_OFFSET))

#ifndef __ASSEMBLY__
#ifdef CONFIG_MAPPED_KERNEL
extern unsigned long phys_to_kernel_offset;
# define __pa_symbol(x) (RELOC_HIDE((unsigned long)(x), 0) - phys_to_kernel_offset)
#else
#define __pa_symbol(x) __pa(RELOC_HIDE((unsigned long)(x), 0))
#endif
#endif

#define pfn_to_kaddr(pfn)	__va((pfn) << PAGE_SHIFT)

#ifdef CONFIG_XEN_FLATMEM
#define pfn_valid(pfn) ((pfn) >= ARCH_PFN_OFFSET && (pfn) < max_mapnr)
#else
#error "Unsupported Memory Layout"
#endif

#define virt_to_page(kaddr)	pfn_to_page(PFN_DOWN(virt_to_phys(kaddr)))
#define virt_addr_valid(kaddr)	pfn_valid(PFN_DOWN(virt_to_phys(kaddr)))

#define VM_DATA_DEFAULT_FLAGS	(VM_READ | VM_WRITE | VM_EXEC | \
				 VM_MAYREAD | VM_MAYWRITE | VM_MAYEXEC)

#define UNCAC_ADDR(addr)	((addr) - PAGE_OFFSET + UNCAC_BASE)
#define CAC_ADDR(addr)		((addr) - UNCAC_BASE + PAGE_OFFSET)

#ifdef CONFIG_LIMITED_DMA
#define WANT_PAGE_VIRTUAL
#endif

#ifndef __ASSEMBLY__

static inline int get_order_from_bytes(paddr_t size)
{
    int order;
    size = (size-1) >> PAGE_SHIFT;
    for ( order = 0; size; order++ )
        size >>= 1;
    return order;
}

static inline int get_order_from_pages(unsigned long nr_pages)
{
    int order;
    nr_pages--;
    for ( order = 0; nr_pages; order++ )
        nr_pages >>= 1;
    return order;
}

#endif

#define PADDR_BITS 40

#include <asm-generic/memory_model.h>
#include <asm-generic/page.h>
#include <asm/xenpage.h>

#endif /* _ASM_PAGE_H */
