#ifndef __ASM_PERCPU_H
#define __ASM_PERCPU_H

#include <xen/config.h>
#include <xen/compiler.h>

#define PERCPU_SHIFT 13
#define PERCPU_SIZE  (1UL << PERCPU_SHIFT)

#ifdef CONFIG_XEN_SMP

/* Separate out the type, so (int[3], foo) works. */
#define DEFINE_PER_CPU(type, name)                      \
    __attribute__((__section__(".data.percpu")))        \
    __typeof__(type) per_cpu__##name

/* var is in discarded region: offset to particular copy we want */
#define per_cpu(var, cpu)  \
    (*RELOC_HIDE(&per_cpu__##var, ((unsigned int)(cpu)) << PERCPU_SHIFT))
//    (*RELOC_HIDE(&per_cpu__##var, ((unsigned int)(0)) << PERCPU_SHIFT))

#define __get_cpu_var(var) \
    (per_cpu(var, smp_processor_id()))

#else

#define DEFINE_PER_CPU(type, name) \
    __typeof__(type) per_cpu__##name

#define per_cpu(var, cpu)			(*((void)(cpu), &per_cpu__##var))
#define __get_cpu_var(var)			per_cpu__##var

#endif

#define DECLARE_PER_CPU(type, name) extern __typeof__(type) per_cpu__##name


#endif /* __ASM_PERCPU_H */
