/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1994 - 2001, 2003 by Ralf Baechle
 * Copyright (C) 1999, 2000, 2001 Silicon Graphics, Inc.
 */
#ifndef _ASM_PGALLOC_H
#define _ASM_PGALLOC_H

#include <xen/mm.h>
#include <xen/debug.h>

extern void pgd_init(unsigned long page);
extern void pmd_init(unsigned long page);
extern void pte_init(unsigned long page);

static inline pgd_t *pgd_alloc(struct mm_struct *mm)
{
	void *pgd;
   
	pgd = alloc_xenheap_page();
	if (pgd != NULL) {
		pgd_init((unsigned long)pgd);
		return (pgd_t *) pgd;
	}
	
	return NULL;
}

static inline void pgd_free(pgd_t *pgd)
{
	free_xenheap_page(pgd);
}

#ifndef CONFIG_XEN_2LEVEL_PGTABLE

static inline pmd_t *pmd_alloc(struct mm_struct *mm,
	unsigned long address)
{
	void *pmd;

	pmd = alloc_xenheap_page();
	if (pmd != NULL) {
		pmd_init((unsigned long)pmd);
		return (pmd_t *) pmd;
	}

	return NULL;
}

static inline void pmd_free(pmd_t *pmd)
{
	free_xenheap_page(pmd);
}

#endif

static inline pte_t *pte_alloc(struct mm_struct *mm,
	unsigned long address)
{
	void *pte;

	pte = alloc_xenheap_page();
	if (pte != NULL) {
		pte_init((unsigned long)pte);
		return (pte_t *) pte;
	}

	return NULL;
}

static inline void pte_free(pte_t *pte)
{
	free_xenheap_page(pte);
}

#endif /* _ASM_PGALLOC_H */
