/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 2003 Ralf Baechle
 */
#ifndef _ASM_PGTABLE_H
#define _ASM_PGTABLE_H

#ifdef CONFIG_XEN_32BIT
#include <asm/pgtable-32.h>
#endif
#ifdef CONFIG_XEN_64BIT
#include <asm/pgtable-64.h>
#endif

#include <asm/io.h>
#include <asm/pgtable-bits.h>

extern unsigned long _page_cachable_default;

#define PAGE_NONE	__pgprot(_CACHE_CACHABLE_NONCOHERENT)
#define PAGE_RW	__pgprot(_PAGE_READ | _PAGE_WRITE | _page_cachable_default)
#define PAGE_READONLY	__pgprot(_PAGE_READ | _page_cachable_default)

#define PAGE_NONE_OWNER	__pgprot(_PAGE_PRESENT | _CACHE_CACHABLE_NONCOHERENT)
#define PAGE_RW_OWNER __pgprot(_PAGE_PRESENT | _PAGE_READ | _PAGE_WRITE | \
				 _page_cachable_default)
#define PAGE_READONLY_OWNER	__pgprot(_PAGE_PRESENT | _PAGE_READ | \
				 _page_cachable_default)

/*
 * Conversion functions: convert a page and protection to a page entry,
 * and a page entry and page directory to the page they refer to.
 */
#define pmd_phys(pmd)		virt_to_phys((void *)pmd_val(pmd))
#define pmd_page(pmd)		(pfn_to_page(pmd_phys(pmd) >> PAGE_SHIFT))

/*
 * @doc: pmd_page_vaddr() might have to change should page
 *       tables be ever changed to capture physical addresses
 */
#define pmd_page_vaddr(pmd)	pmd_val(pmd)

#define pte_none(pte)		(!(pte_val(pte) & ~_PAGE_GLOBAL))

#define pte_present(pte)	(pte_val(pte) & _PAGE_PRESENT)
/* Just for compilation! */
// #define pte_user(pte)           (pte_val(pte) & _PAGE_PRESENT)

/*
 * Certain architectures need to do special things when pte's
 * within a page table are directly modified.  Thus, the following
 * hook is made available.
 */
static inline void set_pte(pte_t *ptep, pte_t pteval)
{
	*ptep = pteval;
#if !defined(CONFIG_CPU_R3000) && !defined(CONFIG_CPU_TX39XX)
	if (pte_val(pteval) & _PAGE_GLOBAL) {
		pte_t *buddy = ptep_buddy(ptep);
		/*
		 * Make sure the buddy is global too (if it's !none,
		 * it better already be global)
		 */
		if (pte_none(*buddy))
			pte_val(*buddy) = pte_val(*buddy) | _PAGE_GLOBAL;
	}
#endif
}
#define set_pte_at(mm,addr,ptep,pteval) set_pte(ptep,pteval)


#endif /* _ASM_PGTABLE_H */
