#ifndef _ASM_MACH_NLM_XLP_MMU_H
#define _ASM_MACH_NLM_XLP_MMU_H

#include <xen/percpu.h>
#include <asm/netl/asmmacro.h>

#define PGW_REGS_BLOCK 4

enum {
    PGW_MMU_INFO = 0x10, 
    PGW_PGD_BASES,
    PGW_PGD_SHIFT,
    PGW_PGD_MASK,
    PGW_PUD_SHIFT,
    PGW_PUD_MASK,
    PGW_PMD_SHIFT,
    PGW_PMD_MASK,
    PGW_PTE_SHIFT,
    PGW_PTE_MASK
};

#define PGD 0x8 
#define PUD 0x4 
#define PMD 0x2 
#define PTE 0x1

#define pgw_register_write_w(reg, value) write_control_reg_w(PGW_REGS_BLOCK, reg, value)
#define pgw_register_write_d(reg, value) write_control_reg_dw(PGW_REGS_BLOCK, reg, value)
#define pgw_register_read_w(reg) read_control_reg_w(PGW_REGS_BLOCK, reg)
#define pgw_register_read_d(reg) read_control_reg_dw(PGW_REGS_BLOCK, reg)

#define pgw_print_w(reg) printk(#reg " = 0x%x\n", pgw_register_read_w(reg))

extern void pgwalker_init(void);
extern void dump_pgwalker_config(void);

#define ENTRYLO_PFN_SHIFT 6

/* 
 * These numbers correspond to Cop0 Config6 reg 
 * bit positions 
 */
#define ENABLE_ETLB        0x4
#define ENABLE_128_TLB     0x20
#define ENABLE_PGWALKER    0x8

#define USER_SEG 0
#define SUPERVISOR_SEG 2

#ifdef CONFIG_XEN_64BIT
#define NR_ADDR_SEGMENTS 8
#else
#define NR_ADDR_SEGMENTS 2

#endif /* CONFIG_64BIT */

DECLARE_PER_CPU(unsigned long [NR_ADDR_SEGMENTS], pgd_bases);

static __inline__ void setup_user_pgd(pgd_t *pgd)
{
    if (read_c0_config6() & ENABLE_PGWALKER) {
		/*
		 * XXX: pgd is set in two entries correponding to user (kuseg)
		 *      and supervisor segments, to accommodate domains 
		 *      compiled for either mode. This is not the correct way
		 *      of doing it if address space straddles both segments
		 *      and a different page table has to be set in different
		 *      entries. Currently, the focus is on short cuts that
		 *      do not fundamentally bust the functionality. Will
		 *      revisit this during peace time
		 */
    	get_cpu_var(pgd_bases)[USER_SEG] = (unsigned long) pgd;
    	get_cpu_var(pgd_bases)[SUPERVISOR_SEG] = (unsigned long) pgd;
        put_cpu_var(pgd_bases);
    }
	else {
		printk("must not be here ...\n");
	}
};

#endif
