/************************************************************************

  Copyright 2003-2006 Raza Microelectronics, Inc.(RMI).

  This is a derived work from software originally provided by the external
  entity identified below. The licensing terms and warranties specified in
  the header of the original work apply to this derived work.

  Contribution by RMI:

 *****************************#RMI_1#************************************/

/*
 * This file is subject to the terms and conditions of the GNU General
 * Public License.  See the file "COPYING" in the main directory of this
 * archive for more details.
 *
 * Copyright (C) 2000 - 2001 by Kanoj Sarcar (kanoj@sgi.com)
 * Copyright (C) 2000 - 2001 by Silicon Graphics, Inc.
 * Copyright (C) 2000, 2001, 2002 Ralf Baechle
 * Copyright (C) 2000, 2001 Broadcom Corporation
 */
#ifndef __ASM_SMP_H
#define __ASM_SMP_H

#include <xen/config.h>
#include <xen/kernel.h>
#include <xen/cpumask.h>
#include <asm/current.h>

#include <asm/atomic.h>
#include <asm/thread_info.h>

extern cpumask_t cpu_sibling_map[];
extern cpumask_t cpu_core_map[];

#define raw_smp_processor_id() (current_thread_info()->task->processor)

#ifdef CONFIG_XEN_SMP

extern cpumask_t cpu_online_map;

#define SMP_RESCHEDULE_YOURSELF	0x1	/* XXX braindead */
#define IPI_CALL_FUNCTION	0x2
#define IPI_EVENT_CHECK_VECTOR	0x3

struct call_data_struct {
	void		(*func)(void *);
	void		*info;
	atomic_t	started;
	atomic_t	finished;
	int		wait;
};

/* Map from cpu id to sequential logical cpu number.  This will only
   not be idempotent when cpus failed to come on-line.  */
extern int __cpu_number_map[NR_CPUS];
#define cpu_number_map(cpu)  __cpu_number_map[cpu]

/* The reverse map from sequential logical cpu number to cpu id.  */
extern int __cpu_logical_map[NR_CPUS];
#define cpu_logical_map(cpu)  __cpu_logical_map[cpu]

#define NO_PROC_ID	(-1)

// extern struct call_data_struct *call_data;

extern cpumask_t phys_cpu_present_map;
#ifdef CONFIG_RMI_PHOENIX
extern cpumask_t cpu_possible_map;
#else
#define cpu_possible_map	phys_cpu_present_map
#endif

/*
 * These are defined by the board-specific code.
 */

/*
 * Cause the function described by call_data to be executed on the passed
 * cpu.  When the function has finished, increment the finished field of
 * call_data.
 */
extern void core_send_ipi(int cpu, unsigned int action);

/*
 * Firmware CPU startup hook
 */
extern void prom_boot_secondary(int cpu, struct task_struct *idle);

/*
 *  After we've done initial boot, this function is called to allow the
 *  board code to clean up state, if needed
 */
extern void prom_init_secondary(void);

/*
 * Populate cpu_possible_map before smp_init, called from setup_arch.
 */
extern void plat_smp_setup(void);

/*
 * Called in smp_prepare_cpus.
 */
extern void plat_prepare_cpus(unsigned int max_cpus);

/*
 * Last chance for the board code to finish SMP initialization before
 * the CPU is "online".
 */
extern void prom_smp_finish(void);

/* Hook for after all CPUs are online */
extern void prom_cpus_done(void);

extern void asmlinkage smp_bootstrap(void);

/*
 * this function sends a 'reschedule' IPI to another CPU.
 * it goes straight through and wastes no time serializing
 * anything. Worst case is that we lose a reschedule ...
 */
static inline void smp_send_reschedule(int cpu)
{
	core_send_ipi(cpu, SMP_RESCHEDULE_YOURSELF);
}

extern asmlinkage void smp_call_function_interrupt(void);

extern void smp_init(void);
extern int init_ipi(void);
extern void boot_secondary(void);

#endif /* CONFIG_XEN_SMP */

#endif /* __ASM_SMP_H */
