/************************************************************************

  Copyright 2003-2006 Raza Microelectronics, Inc.(RMI).

  This is a derived work from software originally provided by the external
  entity identified below. The licensing terms and warranties specified in
  the header of the original work apply to this derived work.

  Contribution by RMI: 

  *****************************#RMI_1#************************************/

/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1994, 95, 96, 97, 98, 99, 2003, 06 by Ralf Baechle
 * Copyright (C) 1996 by Paul M. Antoine
 * Copyright (C) 1999 Silicon Graphics
 * Kevin D. Kissell, kevink@mips.org and Carsten Langgaard, carstenl@mips.com
 * Copyright (C) 2000 MIPS Technologies, Inc.
 */
#ifndef _ASM_SYSTEM_H
#define _ASM_SYSTEM_H

#include <xen/config.h>
#include <xen/types.h>
#include <xen/irqflags.h>
// #include <xen/sched.h>

#include <asm/addrspace.h>
#include <asm/barrier.h>
#include <asm/cpu-features.h>

#include <asm/war.h>

#define task_thread_info(task) (task)->arch.thread_info

/*
 * switch_to(n) should switch tasks to task nr n, first
 * checking that n isn't the current task, in which case it does nothing.
 */
extern asmlinkage void *resume(void *last, void *next, void *next_ti);

// struct task_struct;

#define switch_to(prev,next)					                \
do {									                        \
	resume(prev, next, task_thread_info(next));	                \
} while(0)

/*
 * On SMP systems, when the scheduler does migration-cost autodetection,
 * it needs a way to flush as much of the CPU's caches as possible.
 *
 * TODO: fill this in!
 */
static inline void sched_cacheflush(void)
{
}


extern void set_handler (unsigned long offset, void *addr, unsigned long len);
extern void set_uncached_handler (unsigned long offset, void *addr, unsigned long len);
extern void *set_vi_handler (int n, void *addr);
extern void *set_except_vector(int n, void *addr);
extern unsigned long ebase;

extern int stop_a_enabled;

/*
 * See include/asm-ia64/system.h; prevents deadlock on SMP
 * systems.
 */
#define __ARCH_WANT_UNLOCKED_CTXSW

#define arch_align_stack(x) (x)

#define nop() \
    asm volatile ( "nop" )

#define watchdog_disable() ((void)0)

static inline int local_irq_is_enabled(void)
{
    unsigned long flags;
    raw_local_save_flags(flags);
    return !!(flags & 0x1); /* EFLAGS_IF */
}

static inline void save_epc(unsigned long *epc)
{
    __asm__ __volatile__(
		".set push\n"
		".set noreorder\n"
		"mfc0 %0, $14\n" 
		".set pop\n"
		:"=r"(*epc));
}

#define	OPT_CONSOLE_STR "com1"

#endif /* _ASM_SYSTEM_H */
